package re.lex;

import re.parser.sym;
import re.error.*;

%%
%class Lexer
%public
%cup
%line
%column
 
%{

  Token create(int sym, Object value) {
    return new Token(sym, value, yyline+1);
  }
  
  Token create(int sym) {
    return create(sym, yytext());
  }

%}

WhiteSpace =  [ \t\f\n]

Comment = {TraditionalComment} | {EndOfLineComment}
TraditionalComment   = "/*" [^*] ~"*/" | "/*" "*"+ "/"
EndOfLineComment     = "//" .* "\n"

%%

"let"          { return create(sym.LET); }
"in"           { return create(sym.IN); }
"@"            { return create(sym.EPSILON); }

[a-z]          { return create(sym.LETTER, new Character(yytext().charAt(0))); }
[A-Z][a-z]*    { return create(sym.ID); }

"("            { return create(sym.OPAREN); }
")"            { return create(sym.CPAREN); }
"["            { return create(sym.OBRACKET); }
"]"            { return create(sym.CBRACKET); }
"="            { return create(sym.EQ); }
"."            { return create(sym.DOT); }
"+"            { return create(sym.PLUS); }
"*"            { return create(sym.STAR); }
"|"            { return create(sym.BAR); }
"?"            { return create(sym.QUESTION); }
  
{Comment}      { /* ignore */ }
{WhiteSpace}   { /* ignore */ }

.              { throw new LexicalError("Illegal character '" + yytext() + "'"); }
